var Immutable, addressRunner, fromJS, makeAddress, o, parseAddress, parsePathSegments, parseQuery, routerBase, stringifyQuery, trimSlash;

Immutable = require('immutable');

fromJS = Immutable.fromJS;

o = Immutable.Map();

trimSlash = function(chunk) {
  if (chunk.substr(0, 1) === '/') {
    return trimSlash(chunk.substr(1));
  } else if (chunk.substr(-1) === '/') {
    return trimSlash(chunk.substr(0, chunk.length - 1));
  } else {
    return chunk;
  }
};

parseQuery = function(data, chunks) {
  var chunk, key, pieces, value;
  if (chunks.size === 0) {
    return data;
  } else {
    chunk = chunks.first();
    pieces = chunk.split('=');
    key = pieces[0], value = pieces[1];
    key = decodeURIComponent(key);
    value = decodeURIComponent(value);
    return parseQuery(data.set(key, value), chunks.slice(1));
  }
};

routerBase = Immutable.fromJS({
  name: null,
  data: {},
  query: {},
  router: null
});

parsePathSegments = function(pathSegments, routes, theQuery) {
  var argsTemplate, childRouter, data, nextPathSegments, pathName, restOfSegments;
  if (pathSegments.size === 0) {
    return routerBase.set('name', 'home').set('query', theQuery);
  } else {
    pathName = pathSegments.get(0);
    if (routes.has(pathName)) {
      argsTemplate = routes.get(pathName);
      restOfSegments = pathSegments.rest();
      if (restOfSegments.size < argsTemplate.size) {
        return routerBase.set('name', '404').set('query', theQuery);
      } else {
        data = argsTemplate.reduce(function(acc, argName, idx) {
          return acc.set(argName, restOfSegments.get(idx));
        }, o);
        nextPathSegments = restOfSegments.slice(argsTemplate.size);
        if (nextPathSegments.size > 0) {
          childRouter = parsePathSegments(nextPathSegments, routes, theQuery);
        } else {
          childRouter = null;
        }
        if (childRouter != null) {
          return routerBase.set('name', pathName).set('data', data).set('router', childRouter);
        } else {
          return routerBase.set('name', pathName).set('data', data).set('query', theQuery);
        }
      }
    } else {
      return routerBase.set('name', '404').set('query', theQuery);
    }
  }
};

parseAddress = function(address, routes) {
  var chunkPath, chunkQuery, pathSegments, ref, thePath, theQuery;
  ref = address.split('?'), chunkPath = ref[0], chunkQuery = ref[1];
  chunkQuery = chunkQuery || '';
  chunkPath = trimSlash(chunkPath);
  thePath = chunkPath.length > 0 ? chunkPath.split('/') : [];
  pathSegments = Immutable.fromJS(thePath.map(decodeURIComponent));
  if (chunkQuery.length > 0) {
    theQuery = parseQuery(o, Immutable.fromJS(chunkQuery.split('&')));
  } else {
    theQuery = o;
  }
  return parsePathSegments(pathSegments, routes, theQuery);
};

stringifyQuery = function(query) {
  var stringQuery;
  return stringQuery = query.filter(function(value, key) {
    return value != null;
  }).map(function(value, key) {
    key = encodeURIComponent(key);
    value = encodeURIComponent(value);
    return key + "=" + value;
  }).join('&');
};

addressRunner = function(acc, router, routes, query) {
  var args, argsTemplate, nextAcc, nextQuery, nextRouter, pieces, queryPart, routerName;
  if (router == null) {
    queryPart = stringifyQuery(query);
    if (queryPart.length > 0) {
      if (acc.length > 0) {
        return acc + "?" + queryPart;
      } else {
        return queryPart;
      }
    } else {
      return acc;
    }
  } else {
    routerName = router.get('name');
    if (routes.has(routerName)) {
      argsTemplate = routes.get(routerName);
      nextQuery = router.get('query') || o;
      if (routerName === 'home' && argsTemplate.size === 0) {
        return addressRunner(acc, null, routes, nextQuery);
      } else {
        args = argsTemplate.map(function(argName) {
          return router.get('data').get(argName);
        });
        pieces = args.unshift(routerName);
        nextAcc = acc + "/" + (pieces.join('/'));
        nextRouter = router.get('router');
        return addressRunner(nextAcc, nextRouter, routes, nextQuery);
      }
    } else {
      return acc + "/404";
    }
  }
};

makeAddress = function(router, routes) {
  return addressRunner('', router, routes, router.get('query'));
};

exports.trimSlash = trimSlash;

exports.parseQuery = parseQuery;

exports.makeAddress = makeAddress;

exports.parseAddress = parseAddress;
